#!/bin/sh
#
# irqset
# Shell script for setting and retrieving an interrupt's CPU affinity
#
# Robert Love <rml@tech9.net>	28 April 200
#
# IRQ affinity is exported via a proc interface to /proc/irq/<no>/smp_affinity
# Only root can read or write this value.  The value is a bitmask of allowed
# CPUs.  You can not turn off all CPUs.  If an interrupt controller does not
# support interrupt affinity the bitmask will not change from the default of
# 0xffffffff.
#
# This is licensed under the GNU GPL v2.0.  See COPYING.
# Copyright (C) 2002 Robert Love

VER="0.0.5"
DIR="/proc/irq"

if [ "$1" == "--help" ]; then
	echo "irqset version ${VER}"
	echo "usage: $0 [mask] [interrupt]"
	echo "If no parameters are given, print current affinity CPU masks."
	echo "Report bugs and send patches to <rml@tech9.net>"
	exit 0
fi

if [ $UID != 0 ]; then
	echo "Sorry, you must be superuser to access interrupt affinity"
	exit 1
fi

if [ ! -d /proc ]; then
	echo "Interrupt affinity requires /proc and it is not mounted!"
	exit 1
fi

# if smp_affinity does not exist for irq 0 or 1, assume it is not supported
if [ ! -f ${DIR}/0/smp_affinity -o ! -f ${DIR}/1/smp_affinity ]; then
	echo "Interrupt affinity is not supported on this system!  Either your"
	echo "kernel is too old or (most likely) this system is not SMP."
	exit 1
fi

if [ $2 ]; then
	echo -e -n "interrupt ${1}'s current CPU mask:\t"
	echo "0x`cat ${DIR}/${1}/smp_affinity`"
	echo $2 > ${DIR}/${1}/smp_affinity
	echo -e -n "interrupt ${1}'s new CPU mask:\t"
	echo "0x`cat ${DIR}/${1}/smp_affinity`"
	exit 0
fi

echo -e "IRQ\tCPU Mask\n"

for irq in ${DIR}/*/; do
	no=`echo $irq =" " | awk -F / '{print $4}'` 
	echo -e "$no\t0x`cat ${irq}smp_affinity`"
done

exit 0
